@@ -11,6 +11,7 @@ import os
 import threading
 import logging.config
 import datetime
+from collections import defaultdict
 
 
 try:
@@ -82,8 +83,9 @@ logging.config.dictConfig(LOGGING_CONFIG)
 COMMAND_GET_ALL_COMMITS_SHA = "git log {} --pretty=format:%H"
 COMMAND_COMMIT = "git show --reverse --first-parent --raw --numstat --abbrev=40 --full-index -p -M --pretty=format:'Commit:\t%H%nDate:\t%ai%nTree:\t%T%nParents:\t%P%nAuthor:\t%an\t%ae\t%ai%nCommitter:\t%cn\t%ce\t%ci%nMessage:\t%s%n' {}"
 COMMAND_COMMIT_BRANCH = "git branch --contains {}"
-COMMAND_COMMIT_FILE_BLAME = u"git blame {}^ -L {},{} -- {}"
-COMMAND_COMMIT_FILE_BLAME_FIX = u"git log --pretty=%H -1 {}^ -- {}"
+COMMAND_COMMIT_FILE_BLAME = "git blame {}^ -L {},{} -- {}"
+COMMAND_COMMIT_FILE_BLAME_FIX = "git log --pretty=%H -1 {}^ -- {}"
+COMMAND_REMOTE_URL = "git config --get remote.origin.url"
 
 DEBUG = True
 COMMIT_COUNT = 10
@@ -97,6 +99,290 @@ RE_COMMIT_DIFF = re.compile(
     r"""^diff[ ]--git[ ](?P<a_path_fallback>"?a/.+?"?)[ ](?P<b_path_fallback>"?b/.+?"?)\n(?:^old[ ]mode[ ](?P<old_mode>\d+)\n^new[ ]mode[ ](?P<new_mode>\d+)(?:\n|$))?(?:^similarity[ ]index[ ]\d+%\n^rename[ ]from[ ](?P<rename_from>.*)\n^rename[ ]to[ ](?P<rename_to>.*)(?:\n|$))?(?:^new[ ]file[ ]mode[ ](?P<new_file_mode>.+)(?:\n|$))?(?:^deleted[ ]file[ ]mode[ ](?P<deleted_file_mode>.+)(?:\n|$))?(?:^index[ ](?P<a_blob_id>[0-9A-Fa-f]+)\.\.(?P<b_blob_id>[0-9A-Fa-f]+)[ ]?(?P<b_mode>.+)?(?:\n|$))?(?:^---[ ](?P<a_path>[^\t\n\r\f\v]*)[\t\r\f\v]*(?:\n|$))?(?:^\+\+\+[ ](?P<b_path>[^\t\n\r\f\v]*)[\t\r\f\v]*(?:\n|$))?""",
     re.VERBOSE | re.MULTILINE)
 
+REPOSITORY_NAME = ''
+
+class BasePlatform(object):
+    FORMATS = {
+        'ssh': r"%(_user)s@%(host)s:%(repo)s.git",
+        'ssh2': r"ssh://(_user)s@%(host)s:%(port)s%(path)s%(repo)s.git",
+        'http': r"http://%(host)s/%(repo)s.git",
+        'https': r"http://%(host)s/%(repo)s.git",
+        'git': r"git://%(host)s/%(repo)s.git"
+    }
+
+    PATTERNS = {
+        'ssh': r"(?P<_user>.+)s@(?P<domain>.+)s:(?P<repo>.+)s.git",
+        'ssh2': r"(ssh:\/\/)?(?P<_user>.+)@(?P<domain>.+):(?P<port>\d{2,5})(?P<path>\/(.+(\/))?)(?P<repo>[^\/\d].+).git",
+        'http': r"http://(?P<domain>.+)s/(?P<repo>.+)s.git",
+        'https': r"http://(?P<domain>.+)s/(?P<repo>.+)s.git",
+        'git': r"git://(?P<domain>.+)s/(?P<repo>.+)s.git"
+    }
+
+    # None means it matches all domains
+    DOMAINS = None
+    DEFAULTS = {}
+
+    def __init__(self):
+        # Precompile PATTERNS
+        self.COMPILED_PATTERNS = dict(
+            (proto, re.compile(regex))
+            for proto, regex in self.PATTERNS.items()
+        )
+
+        # Supported protocols
+        self.PROTOCOLS = self.PATTERNS.keys()
+
+
+class BitbucketPlatform(BasePlatform):
+    PATTERNS = {
+        'https': r'https://(?P<_user>.+)@(?P<domain>.+)/(?P<owner>.+)/(?P<repo>.+).git',
+        'ssh': r'git@(?P<domain>.+):(?P<owner>.+)/(?P<repo>.+).git'
+    }
+    FORMATS = {
+        'https': r'https://%(owner)s@%(domain)s/%(owner)s/%(repo)s.git',
+        'ssh': r'git@%(domain)s:%(owner)s/%(repo)s.git'
+    }
+    DOMAINS = ('bitbucket.org',)
+    DEFAULTS = {
+        '_user': 'git'
+    }
+
+
+class GitHubPlatform(BasePlatform):
+    PATTERNS = {
+        'https': r'https://(?P<domain>.+)/(?P<owner>.+)/(?P<repo>.+).git',
+        'ssh': r'git@(?P<domain>.+):(?P<owner>.+)/(?P<repo>.+).git',
+        'git': r'git://(?P<domain>.+)/(?P<owner>.+)/(?P<repo>.+).git',
+    }
+    FORMATS = {
+        'https': r'https://%(domain)s/%(owner)s/%(repo)s.git',
+        'ssh': r'git@%(domain)s:%(owner)s/%(repo)s.git',
+        'git': r'git://%(domain)s/%(owner)s/%(repo)s.git'
+    }
+    DOMAINS = ('github.com', 'gist.github.com',)
+    DEFAULTS = {
+        '_user': 'git'
+    }
+
+
+class GitLabPlatform(BasePlatform):
+    PATTERNS = {
+        'https': r'https://(?P<domain>.+)/(?P<owner>.+)/(?P<repo>.+).git',
+        'ssh': r'git@(?P<domain>.+):(?P<owner>.+)/(?P<repo>.+).git',
+        'git': r'git://(?P<domain>.+)/(?P<owner>.+)/(?P<repo>.+).git',
+    }
+    FORMATS = {
+        'https': r'https://%(domain)s/%(owner)s/%(repo)s.git',
+        'ssh': r'git@%(domain)s:%(owner)s/%(repo)s.git',
+        'git': r'git://%(domain)s/%(owner)s/%(repo)s.git'
+    }
+    DEFAULTS = {
+        '_user': 'git'
+    }
+
+
+PLATFORMS = (
+    # name -> Platform object
+    ('github', GitHubPlatform()),
+    ('bitbucket', BitbucketPlatform()),
+    ('gitlab', GitLabPlatform()),
+
+    # Match url
+    ('base', BasePlatform()),
+)
+
+
+PLATFORMS_MAP = dict(PLATFORMS)
+
+
+SUPPORTED_ATTRIBUTES = (
+    'domain',
+    'repo',
+    'owner',
+    '_user',
+    'port',
+
+    'path',
+
+    'url',
+    'platform',
+    'protocol',
+)
+
+
+def parse_repo_url(url, check_domain=True):
+    # Values are None by default
+    parsed_info = defaultdict(lambda: None)
+    parsed_info['port'] = ''
+    parsed_info['path'] = ''
+
+    # Defaults to all attributes
+    map(parsed_info.setdefault, SUPPORTED_ATTRIBUTES)
+
+    for name, platform in PLATFORMS:
+        for protocol, regex in platform.COMPILED_PATTERNS.items():
+            # Match current regex against URL
+            match = regex.match(url)
+
+            # Skip if not matched
+            if not match:
+                # print("[%s] URL: %s dit not match %s" % (name, url, regex.pattern))
+                continue
+
+            # Skip if domain is bad
+            domain = match.group('domain')
+            # print('[%s] DOMAIN = %s' % (url, domain,))
+            if check_domain:
+                if platform.DOMAINS and not(domain in platform.DOMAINS):
+                    # print("domain: %s not in %s" % (domain, platform.DOMAINS))
+                    continue
+
+            # Get matches as dictionary
+            matches = match.groupdict()
+
+            # Update info with matches
+            parsed_info.update(matches)
+
+            # add in platform defaults
+            parsed_info.update(platform.DEFAULTS)
+
+            # Update info with platform info
+            parsed_info.update({
+                'url': url,
+                'platform': name,
+                'protocol': protocol,
+            })
+            return parsed_info
+
+    # Empty if none matched
+    return parsed_info
+
+
+REQUIRED_ATTRIBUTES = (
+    'domain',
+    'repo',
+)
+
+
+class GitUrlParsed(object):
+    def __init__(self, parsed_info):
+        self._parsed = parsed_info
+
+        # Set parsed objects as attributes
+        for k, v in parsed_info.items():
+            setattr(self, k, v)
+
+    def _valid_attrs(self):
+        return all([
+            getattr(self, attr, None)
+            for attr in REQUIRED_ATTRIBUTES
+        ])
+
+    @property
+    def valid(self):
+        return all([
+            self._valid_attrs(),
+        ])
+
+    @property
+    def _platform_obj(self):
+        return PLATFORMS_MAP[self.platform]
+
+    ##
+    # Alias properties
+    ##
+    @property
+    def host(self):
+        return self.domain
+
+    @property
+    def user(self):
+        if hasattr(self, '_user'):
+            return self._user
+
+        return self.owner
+
+    ##
+    # Format URL to protocol
+    ##
+    def format(self, protocol):
+        return self._platform_obj.FORMATS[protocol] % self._parsed
+
+    ##
+    # Normalize
+    ##
+    @property
+    def normalized(self):
+        return self.format(self.protocol)
+
+    ##
+    # Rewriting
+    ##
+    @property
+    def url2ssh(self):
+        return self.format('ssh')
+
+    @property
+    def url2http(self):
+        return self.format('http')
+
+    @property
+    def url2https(self):
+        return self.format('https')
+
+    @property
+    def url2git(self):
+        return self.format('git')
+
+    # All supported Urls for a repo
+    @property
+    def urls(self):
+        return dict(
+            (protocol, self.format(protocol))
+            for protocol in self._platform_obj.PROTOCOLS
+        )
+
+    ##
+    # Platforms
+    ##
+    @property
+    def github(self):
+        return self.platform == 'github'
+
+    @property
+    def bitbucket(self):
+        return self.platform == 'bitbucket'
+
+    @property
+    def friendcode(self):
+        return self.platform == 'friendcode'
+
+    @property
+    def assembla(self):
+        return self.platform == 'assembla'
+
+    @property
+    def gitlab(self):
+        return self.platform == 'gitlab'
+
+    ##
+    # Get data as dict
+    ##
+    @property
+    def data(self):
+        return dict(self._parsed)
+
+
+def get_repo_name(dflt='UNKNOWN'):
+    repo_name = dflt
+    remote_repo = execute(COMMAND_REMOTE_URL)
+    try:
+        r = GitUrlParsed(parse_repo_url(url=remote_repo, check_domain=True))
+        repo_name = r.repo
+    except Exception as exc:
+        logging.exception(exc, exc_info=True)
+    return repo_name
+
 
 class ThreadWithReturnValue(threading.Thread):
     def __init__(self, group=None, target=None, name=None, args=(), kwargs={}, Verbose=None):
@@ -154,6 +440,7 @@ def _pick_best_path(path_match, rename_match, path_fallback_match):
 
 
 def _parse_numstats(text):
+    repo_name = REPOSITORY_NAME
     hsh = {"total": {"additions": 0, "deletions": 0, "changes": 0, "total": 0, "files": 0}, "files": {}}
     for line in text.splitlines():
 
@@ -176,13 +463,14 @@ def _parse_numstats(text):
         hsh["total"]["changes"] += insertions + deletions
         hsh["total"]["total"] += insertions + deletions
         hsh["total"]["files"] += 1
-        hsh["files"][filename.strip()] = {"filename": filename.strip(), "additions": insertions, "deletions": deletions,
+        hsh["files"][f'{repo_name}/' + filename.strip()] = {"filename": f'{repo_name}/' + filename.strip(), "additions": insertions, "deletions": deletions,
                                           "changes": insertions + deletions}
     return (hsh["total"], hsh["files"])
 
 
 def _parse_stats(text):
     diffs = dict()
+    repo_name = REPOSITORY_NAME
 
     for line in text.splitlines():
         try:
@@ -239,15 +527,20 @@ def _parse_stats(text):
             status = "renamed"
 
         diff = dict(
-            filename=filename, previous_filename=previous_filename, sha=sha,
-            status=status, a_path=a_path, b_path=b_path, a_blob_id=a_blob_id,
+            filename=f'{repo_name}/' + filename if filename else filename,
+            previous_filename=f'{repo_name}/' + previous_filename if previous_filename else previous_filename,
+            sha=sha,
+            status=status,
+            a_path=f'{repo_name}/' + a_path if a_path else a_path,
+            b_path=f'{repo_name}/' + b_path if b_path else b_path,
+            a_blob_id=a_blob_id,
             a_blob=a_blob, b_blob_id=b_blob_id, b_blob=b_blob,
             a_mode=old_mode, b_mode=new_mode, new_file=new_file,
             deleted_file=deleted_file, rename_from=rename_from, rename_to=rename_to,
             change_type=change_type, score=score, patch=""
         )
 
-        diffs[filename] = diff
+        diffs[f'{repo_name}/' + filename if filename else filename] = diff
 
     return diffs
 
@@ -255,6 +548,7 @@ def _parse_stats(text):
 def _parse_patch(text):
     diffs = list()
     previous_header = None
+    repo_name = REPOSITORY_NAME
 
     for header in RE_COMMIT_DIFF.finditer(text):
         a_path_fallback, b_path_fallback, old_mode, new_mode, \
@@ -301,8 +595,13 @@ def _parse_patch(text):
             status = "modified"
 
         diff = dict(
-            filename=filename, previous_filename=previous_filename, sha=sha,
-            status=status, a_path=a_path, b_path=b_path, a_blob_id=a_blob_id,
+            filename=f'{repo_name}/' + filename if filename else filename,
+            previous_filename=f'{repo_name}/' + previous_filename if previous_filename else previous_filename,
+            sha=sha,
+            status=status,
+            a_path=f'{repo_name}/' + a_path if a_path else a_path,
+            b_path=f'{repo_name}/' + b_path if b_path else b_path,
+            a_blob_id=a_blob_id,
             a_blob=a_blob, b_blob_id=b_blob_id, b_blob=b_blob,
             a_mode=a_mode and a_mode,
             b_mode=b_mode and b_mode,
@@ -325,6 +624,13 @@ def _parse_patch(text):
     return dict_diffs
 
 
+def _parse_person(text):
+    (person_name, person_email, person_date) = text.split("\t")
+    person_date = person_date.split(" ")
+    person_date = "{}T{}{}".format(person_date[0], person_date[1], person_date[2])
+    return {"name": person_name, "email": person_email, "date": person_date}
+
+
 def execute(commandLine):
     process = subprocess.Popen(commandLine, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 
@@ -357,13 +663,6 @@ def get_commits_sha(start, number, branch):
     return commits_sha
 
 
-def _parse_person(text):
-    (person_name, person_email, person_date) = text.split("\t")
-    person_date = person_date.split(" ")
-    person_date = "{}T{}{}".format(person_date[0], person_date[1], person_date[2])
-    return {"name": person_name, "email": person_email, "date": person_date}
-
-
 def request(url, token, data, event):
     headers = {"Content-Type": "application/json",
                 "X-Git-Event": event,
@@ -377,7 +676,7 @@ def request(url, token, data, event):
         if resp.status_code == 401:
             logging.error('Could not verify, please check it and try again.')
             sys.exit(1)
-    except Exception:
+    except Exception as e:
         logging.error('Can\'t not get a connection to the server, please check your url try again.')
         result = (None, None)
     return result
@@ -395,7 +694,7 @@ def get_project_id(base_url, project_name, token):
         if resp.status_code == 401:
             logging.error('Could not verify your token, please check it and try again.')
             sys.exit(1)
-    except Exception:
+    except Exception as e:
         logging.error('Can\'t not get a connection to the server, please check your url or token and try again.')
         sys.exit(1)
     return resp.text
@@ -430,15 +729,26 @@ def get_commit_branch(sha):
     return list(set(branch_list))
 
 
+# exclude = set(['.git', '$tf'])
+# allFileNames = []
+# DirectoryPath = '.'
+# def get_file_tree():
+#     for path, subdirs, files in os.walk(DirectoryPath):
+#         subdirs[:] = [sub for sub in subdirs if sub not in exclude]
+#         for name in files:
+#             allFileNames.append(os.path.join(path, name).lstrip('.').lstrip('/'))
+#     return allFileNames
 exclude = set(['.git', '$tf'])
-allFileNames = []
-DirectoryPath = '.'
 def get_file_tree():
-    for path, subdirs, files in os.walk(DirectoryPath):
+    files_paths = []
+    repo_name = get_repo_name()
+    for path, subdirs, files in os.walk('.'):
         subdirs[:] = [sub for sub in subdirs if sub not in exclude]
         for name in files:
-            allFileNames.append(os.path.join(path, name).lstrip('.').lstrip('/'))
-    return allFileNames
+            file_path = os.path.join(path, name).lstrip('.').lstrip('/').lstrip('\\')
+            repo_file_path = f'{repo_name}/' + file_path
+            files_paths.append(repo_file_path)
+    return files_paths
 
 
 def get_parent_commit(sha_parent, blame=False):
@@ -760,7 +1070,7 @@ def get_commit(sha, blame=False):
     return commit
 
 
-def wrap_push_event(ref, commits, file_tree):
+def wrap_push_event(ref, commits, file_tree, repo_name=''):
     try:
         data = {
             "before": commits[0]["sha"],
@@ -772,6 +1082,7 @@ def wrap_push_event(ref, commits, file_tree):
             "size": len(commits),
             "head_commit": commits[-1],
             "file_tree": file_tree,
+            "repo_name": repo_name,
         }
         return json.dumps(data)
     except Exception as e:
@@ -779,56 +1090,14 @@ def wrap_push_event(ref, commits, file_tree):
         return json.dumps({})
 
 
-parser = argparse.ArgumentParser(description='Sync a number of commits before a specific commit')
-
-
-parser.add_argument('--url', type=str, required=True,
-                    help='Enter your organization url')
-parser.add_argument('--project', type=str, required=True,
-                    help='Enter project name')
-parser.add_argument('--token', type=str, required=True,
-                    help='The API key to communicate with API')
-parser.add_argument('--start', type=str, required=True,
-                    help='Enter the commit that would be the starter')
-parser.add_argument('--number', type=int,
-                    help='Enter the number of commits that would be returned')
-parser.add_argument('--branch', type=str, required=True,
-                    help='Enter the explicity branch to process commit')
-parser.add_argument('--blame', action='store_true',
-                    help='Choose to commit revision of each line or not')
-parser.add_argument('--debug', action='store_true',
-                    help='Write data of commits to json file')
-
-
-args = parser.parse_args()
-
-base_url = args.url.rstrip('/')
-project = args.project
-token = args.token
-start = args.start
-number = args.number if args.number else 10
-branch = args.branch
-blame = args.blame
-debug = args.debug
-
-
-project_id_data = json.loads(get_project_id(base_url=base_url, project_name=project, token=token))
-if 'project_id' in project_id_data:
-    project_id = project_id_data['project_id']
-    url = base_url + '/api/ssh_v2/hook/{}/'.format(project_id)
-elif 'error' in project_id_data:
-    logging.error('Project not found')
-    sys.exit(1)
-
-
-def performPush(url, token, start, number, branch, blame):
+def performPush(url, token, start, number, branch, blame, repo_name=''):
     sha_list = get_commits_sha(start=start, number=number, branch=branch)
     commits = list()
     for sha in sha_list:
         commit = get_commit(sha=sha, blame=blame)
         commits.append(commit)
     file_tree = get_file_tree()
-    data = wrap_push_event(ref=branch, commits=commits, file_tree=file_tree)
+    data = wrap_push_event(ref=branch, commits=commits, file_tree=file_tree, repo_name=repo_name)
     # with open('./data.txt', 'w') as f:
     #     f.write(data)
     if debug:
@@ -841,7 +1110,7 @@ def performPush(url, token, start, number, branch, blame):
 
 def gittoappsurify():
     logging.info('Started syncing commits to {}'.format(base_url))
-    performPush(url=url, token=token, start=start, number=number, branch=branch, blame=blame)
+    performPush(url=url, token=token, start=start, number=number, branch=branch, blame=blame, repo_name=repo_name)
     logging.info('Successfully synced commits to {}'.format(base_url))
     logging.info('Start commit: {}'.format(start))
     logging.info('Number of commit(s): {}'.format(number))
@@ -849,4 +1118,54 @@ def gittoappsurify():
 #example usage gittoappsurify --url "https://demo.appsurify.com/" --project "GitScript" --token "MTU6ZW9FZUxhcXpMZU9CdGZZVmZ4U3BFM3g5MmhVcDl5ZmQzampUWEM1SWRfNA" --start "a3b8cad7c079beab89e8fba3f497fe5a1fff367d" --branch "master"
 
 if __name__ == '__main__':
-    gittoappsurify()
\ No newline at end of file
+    parser = argparse.ArgumentParser(description='Sync a number of commits before a specific commit')
+
+    parser.add_argument('--url', type=str, required=True,
+                        help='Enter your organization url')
+    parser.add_argument('--project', type=str, required=True,
+                        help='Enter project name')
+    parser.add_argument('--token', type=str, required=True,
+                        help='The API key to communicate with API')
+    parser.add_argument('--start', type=str, required=True,
+                        help='Enter the commit that would be the starter')
+    parser.add_argument('--number', type=int,
+                        help='Enter the number of commits that would be returned')
+    parser.add_argument('--branch', type=str, required=True,
+                        help='Enter the explicity branch to process commit')
+    parser.add_argument('--blame', action='store_true',
+                        help='Choose to commit revision of each line or not')
+    parser.add_argument('--debug', action='store_true',
+                        help='Write data of commits to json file')
+    parser.add_argument('--repo_name', type=str, required=False, default='',
+                        help='Define repository name')
+    parser.add_argument('--auto_repo_name', action='store_true', default=False,
+                        help='Use Git remote as repository name.')
+
+    args = parser.parse_args()
+
+    base_url = args.url.rstrip('/')
+    project = args.project
+    token = args.token
+    start = args.start
+    number = args.number if args.number else 100
+    branch = args.branch
+    blame = args.blame
+    debug = args.debug
+
+    repo_name = args.repo_name
+    auto_repo_name = args.auto_repo_name
+
+    if auto_repo_name:
+        REPOSITORY_NAME = get_repo_name()
+    else:
+        REPOSITORY_NAME = repo_name
+
+    project_id_data = json.loads(get_project_id(base_url=base_url, project_name=project, token=token))
+    if 'project_id' in project_id_data:
+        project_id = project_id_data['project_id']
+        url = base_url + '/api/ssh_v2/hook/{}/'.format(project_id)
+    elif 'error' in project_id_data:
+        logging.error('Project not found')
+        sys.exit(1)
+
+    gittoappsurify()
